using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using System.IO;
using System.Reflection;
using System.Collections.Specialized;
using System.Xml.Serialization;
using System.Collections;
using System.Web;

namespace Futureproofed.Settings.Provider
{
    public class IniSettingsProvider : LocalFileSettingsProvider
    {
        private static string sIniFileName = "";

        public override string ApplicationName
        {
            get { return base.ApplicationName; }
            set { base.ApplicationName = value; }
        }

        public override string Name
        {
            get {return "IniSettingsProvider";}
        }

        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
        {
            return;
        }

        private static String GetIniFileName(SettingsContext context)
        {
            if (sIniFileName != "")
                return sIniFileName;

            if (context.Contains("IniFileName"))
            {
                sIniFileName = context["IniFileName"].ToString();
                return sIniFileName;
            }

            Assembly asm = Assembly.GetEntryAssembly();
            String fName = asm.Location.ToLower();
            sIniFileName = fName + ".ini";
            return sIniFileName;
        }

        private static Object DeserializeEncodedXml(String type, String assembly, String encodedXml)
        {
            Assembly asm = null;
            if (assembly == "")
            {
                asm = Assembly.GetEntryAssembly();
            }
            else
            {
                asm = Assembly.Load(assembly);
            }
            XmlSerializer mySerializer = new XmlSerializer(asm.GetType(type));
            String encXml = HttpUtility.UrlDecode(encodedXml);
            String xml = HttpUtility.HtmlDecode(encXml);
            return mySerializer.Deserialize(new StringReader(xml));
        }


        private static String SerializeXml(Type type, Object value)
        {
            XmlSerializer mySerializer = new XmlSerializer(type);
            StringWriter strWriter = new StringWriter();
            mySerializer.Serialize(strWriter, value);
            String strHtml = HttpUtility.HtmlEncode(strWriter.ToString());
            String strUrl = HttpUtility.UrlEncode(strHtml);
            return strUrl;
        }

        private static String GetStringForSettings(SettingsPropertyValue spv)
        {
            if (spv.PropertyValue != null)
            {
                String encXml = SerializeXml(spv.PropertyValue.GetType(), spv.PropertyValue);
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append(spv.Name);
                strBuilder.Append("=");
                strBuilder.Append(encXml);
                strBuilder.Append("!");
                if (spv.Property.PropertyType.Namespace != "")
                {
                    strBuilder.Append(spv.Property.PropertyType.Namespace);
                    strBuilder.Append(".");
                }
                strBuilder.Append(spv.Property.PropertyType.Name);
                strBuilder.Append("!");
                strBuilder.Append(spv.Property.PropertyType.Assembly.GetName().Name);
                return strBuilder.ToString();
            }
            else
            {
                return null;
            }
        }

        public override SettingsPropertyValueCollection GetPropertyValues(SettingsContext context, SettingsPropertyCollection ppc)
        {
            String fName = GetIniFileName(context);
            StringCollection sc = INIWrapper.GetINISection(fName, context["GroupName"].ToString());
            SettingsPropertyValueCollection spvc = new SettingsPropertyValueCollection();
            //Here I need to get the values out of this and stuff into a decent place.Probably a SettingsPropertyCollection?
            //The format of the strings are supposed to be <name> = <value>[!<type>]
            Hashtable ht = new Hashtable();
            foreach (string s in sc)
            {
                String[] sp = s.Split('=');
                String[] spType = sp[1].Split('!');
                ht.Add(sp[0], spType);
            }

            foreach (SettingsProperty sp in ppc)
            {
                SettingsPropertyValue spv = new SettingsPropertyValue(sp);
                if (ht.ContainsKey(sp.Name))
                {
                    String[] arr = (String[])ht[sp.Name];
                    String strValue = arr[0];
                    String strType = arr[1];
                    String strAsm = "";
                    if (arr.Length > 2)
                    {
                        strAsm = arr[2];
                    }
                    spv.PropertyValue = DeserializeEncodedXml(strType, strAsm, strValue);
                    spv.SerializedValue = strValue;
                }
                spvc.Add(spv);
            }
            return spvc;
        }

        public override void SetPropertyValues(SettingsContext context, SettingsPropertyValueCollection ppvc)
        {
            String fName = GetIniFileName(context);
            StringCollection coll = new StringCollection();
            foreach (SettingsPropertyValue spv in ppvc)
            {
                String s = GetStringForSettings(spv);
                if (s != null)
                    coll.Add(s);
            }
            INIWrapper.WriteINISection(fName, context["GroupName"].ToString(), coll);
            //			base.SetPropertyValues(context, ppvc);
        }
    }
}
